%cojoc(at)hotmail.com
%2012/4/9
%2014/12 updated
function [Cfinal C] = clSpectralClusteringNcut(S,k,Nthr)
%[Cfinal C] = clSpectralClusteringNcut(S,k,Nthr)
%Spectural Clustering using normalized cut
%input:
%S: similarity matrix, should be possitive, the larger, the more similiar
%k: #of iterations, should be larger than 1
%Nthr: normalize cut threshold
%output:
%Cfinal: final cluster labels
%C[:,i]: cluster id of each node in ith iteration
    v=1;
    if(min(min(S))<0)
        error('error: similarity should be larger or equal to 0.');
    end
%     if size(S)<10
%         error('error: # of nodes must be more than 10');
%     end
    if nargin==1
        k=double(uint32(log2(size(S,1)/10)));
    end
    if nargin<3
        Nthr=0.4;
    end
    C=ones(size(S,1),k);
    P=ones(size(S,1),k);
    for i=1:k
        if i==1 %initial iteration
            pidx=1;
        else
            pidx=i-1;
        end
        C_uniq=unique(C(:,pidx));
        for j=1:length(C_uniq)
            list=find(C(:,pidx)==C_uniq(j));
            if P(list(1),pidx)==1
                ctmp=BiCut(S(list,list),Nthr);
                if min(ctmp)==0 %failed spliting
                    C(list,i)=1+(C(list,pidx)-1)*2;
                    P(list,i)=0;
                else %bi-partioned
                    C(list,i)=ctmp+(C(list,pidx)-1)*2;
                end
            else %skip bi-partion trying
                C(list,i)=1+(C(list,pidx)-1)*2;
                P(list,i)=0;                
            end
        end
        
        if v
            disp([i,length(unique(C(:,i)))]);
        end
    end
    
    %updated: 2014/12 HB
    clist=unique(C(:,end));
    Cfinal=C(:,end);
    for i=1:length(clist)
        Cfinal(C(:,end)==clist(i))=i;
    end
end

% %get the smoothness of the vector
% function Rate = SmoothRate(vector)
%     base=double(uint16(length(vector)/15));
%     if base<3
%         base=3;
%     end
%     mx=max(vector);
%     mi=min(vector);
%     step=(mx-mi)/base;
%     H=histc(vector,mi:step:mx);
%     H=H(1:end-1);
%     Rate=min(H)/max(H);
% end

%bipartition graph S
%if can split, value in C will be either 1 or 2 to indicate sub-graph
%otherwise, all the value in C will be 1
function C = BiCut(S, Nthr)
    C=zeros(size(S,1),1);
    if(size(S)>10)
        L=GetLaplacian(S);
        eig_notreal=1;
        converge = 10;
        eid=2:min(6,size(L,1));
        while eig_notreal || converge > 0
            [V D flag] = eigs(L, max(eid), 'SA');
            if(sum(~isreal(V))>0)
                disp(['warning: get imaginary eigen vector for eigen vector.']);
                disp(['         will recalculate eigen vector.']);
            else
                eig_notreal=0;
            end
            if flag == 1
                converge=converge-1;
                disp(['warning: did not converge, try again. Trial: ' num2str(converge)]);
            else
                converge = 0;
            end
        end
        if flag == 1
            C(:)=0;
        else
            C=kmeans(V(:,eid),2,'replicates',7);
            ncut_tmp=clGetNcut(find(C==1),find(C==2),S);
    %         ncut_tmp=sum(sum(S(C==1,C==1)))+sum(sum(S(C==2,C==2)));
    %         ncut_tmp=ncut_tmp/sum(sum(S));
            if ncut_tmp<Nthr
                %adjust id of cluster if necessary
                list=1:size(S,1);
                m1=mean(list(C==1));
                m2=mean(list(C==2));
                if m1>m2
                    C(C==1)=3;
                    C(C==2)=1;
                    C(C==3)=2;
                end
            else
                C(:)=0;
            end
        end
    end
end


%compute laplacian
function L = GetLaplacian(mat)
    if size(mat,1) ~= size(mat,2)
        error('error: similarity matrix should be diagnal symetric matrix');
    end
    mat = mat-diag(diag(mat));
    if issparse(mat)
        D = sqrt(sum(mat));
        D(D<1e-10) = 1e-10; %in case of 0 degree point
        D = sparse(1:length(D),1:length(D),1./D);
        L = D*mat*D;
        L = sparse(1:length(D),1:length(D),ones(length(D),1))-L;
        L = (L+L')/2;
    else
        D = sqrt(sum(mat));
        D(D<1e-10) = 1e-10; %in case of 0 degree point
        D = diag(1./D);
        L = D*mat*D;
        L = eye(size(mat,1))-L;
        L = (L+L')/2;
    end
end