%cojoc(at)hotmail.com
%2012/7/14
%Obtain eigenvector for spectral clustering
%input:
%S: 2 dimensional similarity matrix
%VK: #of eigenvectors consider, should be between 2 and #of samples
%output:
%V: eigenvector
function V = clSpectralGetSimMatEigenvector(S,VK)
    if(min(min(min(S)))<0)
        error('error: similarity should be larger or equal to 0.');
    end
    VK=VK+1;
    %%view b
    L=GetLaplacian(S);
    eig_notreal=1;
    while eig_notreal
        [Vec ignore] = eigs(L, VK, 'sa');
        if(sum(~isreal(Vec))>0)
            disp(['warning: get imaginary eigen vector for eigen vector.']);
            disp(['         will recalculate eigen vector.']);
        else
            eig_notreal=0;
        end
    end
    V=Vec(:,2:end); %remove the first constent eigenvector
end

%compute laplacian
function L = GetLaplacian(mat)
    if size(mat,1) ~= size(mat,2)
        error('error: similarity matrix should be diagnal symetric matrix');
    end
    mat = mat-diag(diag(mat));
    type=1; 
    switch type
        case 1
            D = sqrt(sum(mat));
            D(D<1e-10) = 1e-10; %in case of 0 degree point
            D = diag(1./D);
            L = D*mat*D;
            L = eye(size(mat,1))-L;
            L = (L+L')/2;
        case 2
            D=sum(mat);
            D(D<1e-10) = 1e-10; %in case of 0 degree point
            D = diag(1./D);
            L=D*mat;    
            L = eye(size(mat,1))-L;
            L = (L+L')/2;
        case 3
            L = diag(sum(mat))-mat;
        case 4
            D = diag(1./sqrt(sum(mat)));
            L = D*mat*D;
    end
end