%cojoc(at)uga.edu
%http://cs.uga.edu/~hanbo/
%2012

%Single View Groupwise Spectural Training
%%input and output description
%input:
%S: n*n*m matrix, similarity matrix of n nodes of m views
%VK: #of eigenvectors consider, should be between 2 and #of samples
%iter: #of itertation
%output:
%mv: n*n matrix, co-trained matrix of all views
%mvi: n*n*iter matrix, co-trained matrix of each iteration
%nmi: iter+1 vector, normalized mutual information, the larger the more
%     agreement between views. (computation is time consuming)
%sall: iter+1 cells, the trained matrix of each subject of each iteration
%      (will caused a lot of memory, only used for test perpose)
function [mv mvi nmi sall] = clSpectralMultiViewTraining(S,VK,iter)
    if(min(min(min(S)))<0)
        error('error: similarity should be larger or equal to 0.');
    end
    subNum=size(S,3);
    
    sf=S;
    if nargout>3    %won't save if not required
        sall=cell(iter+1,1);
        sall{1}=S;
    end
    if nargout>2
        nmi=zeros(iter+1,1);
        nmi(1)=connGetNormalizedMutualInfo(sf);
    end
    mvi=zeros(size(S,1),size(S,2),iter);
    
    Vall=cell(subNum,1);
    for i=1:iter
        Vmap=zeros(size(S,1));
        %obtain eigenvector of each subject and view
        for subi=1:subNum
            Vall{subi}=clSpectralGetSimMatEigenvector(sf(:,:,subi),VK);
            Vmap=Vmap+Vall{subi}*Vall{subi}';
        end
        
        %map each subject and each view
        %initial vector
        for subi=1:subNum
            tmp=(Vmap-Vall{subi}*Vall{subi}')*sf(:,:,subi); %map to eigenspace
            tmp=tmp+tmp';               %symmentrilize
            tmp(tmp<0)=0;               %set negative value to 0
            tmp=tmp-diag(diag(tmp));    %remove diagnol value
            tmp=clNormalizeMat(tmp);    %normalize matrix
            sf(:,:,subi)=clNormalizeMat(tmp);
        end
        if nargout>3
            sall{i+1}=sf;
        end
        if nargout>2
            nmi(i+1)=connGetNormalizedMutualInfo(sf);
        end
        mvi(:,:,i)=mean(sf,3);
        disp(['iter:' num2str(i)]);
    end
    mv=mean(sf,3);
end