%cojoc(at)uga.edu
%http://cs.uga.edu/~hanbo/
%2012
%2014/12 updated by HB

function [mv1,mv2,I,mvall] = clSpectralPairTraining_corr(S1,S2, k, iter)
%Two View Spectural Training, will return the results before first P-correlation drop
%[mv1 mv2 I mvall] = clSpectralPairTraining_corr(S1,S2, k, iter)
%input:
%S1: 2 dimensonal similarity matrix of view 1
%S2: 2 dimensonal similarity matrix of view 2
%k: #of eigenvectors consider, should be between 2 and #of samples
%iter: #of itertation
%output:
%mv1: cotrained matrix of view 1
%mv2: cotrained matrix of view 2
%I, mvall: for test

    if(min(min(S1))<0)
        error('error: similarity should be larger or equal to 0.');
    end
    if(min(min(S2))<0)
        error('error: similarity should be larger or equal to 0.');
    end
    k=k+1;
    K1=S1;
    K2=S2;
    mvall=cell(3,1);
    mvall{1}=cell(iter+1,1);
    mvall{1}{1}.S=K1;
    mvall{2}=cell(iter+1,1);
    mvall{2}{1}.S=K2;
    co=zeros(iter,1);
    for i=1:iter
        %get eigenvector of each view
        V1=clSpectralGetSimMatEigenvector(K1,k);
        V2=clSpectralGetSimMatEigenvector(K2,k);
        
        %get temporal co-trained sim matrices
        K1=V2*V2'*K1;
        K1=(K1+K1')/2;
        K1(K1<0)=0;
        K1=K1-diag(diag(K1));
        K2=V1*V1'*K2; 
        K2=(K2+K2')/2;
        K2(K2<0)=0;
        K2=K2-diag(diag(K2));
        
        co(i)=corr(K1(:),K2(:));
        
        %save result
        mvall{1}{i+1}.S=K1;
        mvall{1}{i+1}.V=V1;
        mvall{2}{i+1}.S=K2;
        mvall{2}{i+1}.V=V2;
    end
    mvall{3}=co;
    
    [ignore I]=max(co(:));
    mv1=clNormalizeMat(mvall{1}{I+1}.S);
    mv2=clNormalizeMat(mvall{2}{I+1}.S);
end
