%cojoc(at)uga.edu
%http://cs.uga.edu/~hanbo/
%2012/11/11

%demo script for multiview spectural clustering
%this demo will show how clSpectralMultiViewTraining.m works
clear; clc;

%generate simulation data
N=500; CN=7; Csize=ceil(N/CN); N=CN*Csize;
P=randperm(N);
Sim=zeros(N,N,2);
Sim_true=zeros(N,N);
for i=1:CN
    list=P((i-1)*Csize+1:i*Csize);
    Sim_true(list,list) = 1;
    Sim(list,list,1)=rand;  %similarity matrix of view 1
    Sim(list,list,2)=rand;  %similarity matrix of view 2
end
Sim=Sim+rand(N,N,2)-0.5;
Sim(Sim<0)=0;

%perform multiview spectural clustering
K=20; %# of eigenvector considered, smaller K will cause lose of information, larger K will cause noise and low convergance speed
[ignore allsim NMI]=clSpectralMultiViewTraining(Sim,K,15);

%perform clustering
nthr=0.5; %threshold for normalize cut
NMI_max=max(NMI); I=find(NMI==NMI_max)-1;
C=clSpectralClusteringNcut(allsim(:,:,I),7,nthr);

%display results
figure; plot(NMI); title('NMI of each iteration');
C=C(:,7); [ignore P]=sort(C);
list=unique(C);
n=length(list);
rect=[];
x=0;y=0;
for i=1:n
    w=sum(C==list(i));
    rect=[rect; x, y, w, w];
    x=x+w;
    y=y+w;
end
figure;
subplot(2,2,1); imagesc(Sim(P,P,1)); colormap hot;
title('view 1 matrix');
for j=1:size(rect,1)
    rectangle('Position',rect(j,:),'EdgeColor','m','LineWidth',1.1);
end
subplot(2,2,2); imagesc(Sim(P,P,2)); colormap hot;
title('view 2 matrix');
for j=1:size(rect,1)
    rectangle('Position',rect(j,:),'EdgeColor','m','LineWidth',1.1);
end
subplot(2,2,3); imagesc(allsim(P,P,I)); colormap hot;
title('fused connection matrix');
for j=1:size(rect,1)
    rectangle('Position',rect(j,:),'EdgeColor','m','LineWidth',1.1);
end
subplot(2,2,4); imagesc(Sim_true(P,P)); colormap hot;
title('true connection matrix');
for j=1:size(rect,1)
    rectangle('Position',rect(j,:),'EdgeColor','m','LineWidth',1.1);
end
